/*
	File:		ModemDriver.c

	Contains:	xxx put contents here xxx

	Version:	xxx put version here xxx

	Copyright:	 1998-1999 by Apple Computer, Inc., all rights reserved.

	File Ownership:

		DRI:				xxx put dri here xxx

		Other Contact:		xxx put other contact here xxx

		Technology:			xxx put technology here xxx

*/

#include <DriverServices.h>

#include "Modem.h"
#include "ModemDriver.h"
#include "ShimSerialHAL.h"

enum
{
	kUSBv12	=	0x01200000
};

static serPB 		gSerialGlobals;
static USBPB		syncPB, interruptPB, readPB, writePB, stallPB, delayPB;
static UInt16		deadman = 20;
static OSStatus		configerr = noErr;
volatile Boolean	configured = false;
UInt32				usbVersion = 0;
Boolean				gUSBVersionNeedsBulkFixPresent;

/************************************************************************************/
//
//	CheckUSBVersion
//
//	Determines whether it's USB1.2 (less requires doing safe bulk calls)
//
/************************************************************************************/
void	CheckUSBVersion(void)
{
OSStatus	err = 0;

	TraceMessage(0, "\pEntering CheckUSBVersion;g");
	
	err = Gestalt('usbv', (long*)&usbVersion);
		
	if (err == noErr)
		gUSBVersionNeedsBulkFixPresent = (usbVersion < kUSBv12);
}

/************************************************************************************/
//
//	SecondaryUSBBulkRead
//
//	USBBulkRead called at safe secondary int. time (USB 1.1 or earlier)
//
/************************************************************************************/
OSStatus SecondaryUSBBulkRead(void *pb, void *result)
{
	*(OSStatus*)result = USBBulkRead((USBPB*)pb);
	return noErr;
}

/************************************************************************************/
//
//	SecondaryUSBBulkWrite
//
//	USBBulkWrite called at safe secondary int. time (USB 1.1 or earlier)
//
/************************************************************************************/
OSStatus SecondaryUSBBulkWrite(void *pb, void *result)
{
	*(OSStatus*)result = USBBulkWrite((USBPB*)pb);
	return noErr;
}

/************************************************************************************/
//
//	SafeUSBBulkRead
//
//	Determines which version of the bulk read to call.
//
/************************************************************************************/
OSStatus SafeUSBBulkRead(USBPB *pb)
{
	OSStatus	result;

	if (gUSBVersionNeedsBulkFixPresent)
	{
		// Use CallSecondaryInterruptHandler2 to call USBBulkRead if
		// less than USB v1.2 present
		CallSecondaryInterruptHandler2(SecondaryUSBBulkRead, nil, pb, &result);
	}
	else
		result = USBBulkRead(pb);
		
	return result;
}

/************************************************************************************/
//
//	SafeUSBBulkWrite
//
//	Determines which version of the bulk write to call.
//
/************************************************************************************/
OSStatus SafeUSBBulkWrite(USBPB *pb)
{
	OSStatus	result;

	if (gUSBVersionNeedsBulkFixPresent)
	{
		// Use CallSecondaryInterruptHandler2 to call USBBulkWrite if
		// less than USB v1.2 present
		CallSecondaryInterruptHandler2(SecondaryUSBBulkWrite, nil, pb, &result);
	}
	else
		result = USBBulkWrite(pb);
		
	return result;
}

/************************************************************************************/
//
//	USBStatus
//
//	Determines which status call to make.
//
/************************************************************************************/

void USBStatus(UInt32 level, USBDeviceRef ref, void *pointer, UInt32 value)
{
	
	if (usbVersion < kUSBv12)
	{
		USBExpertStatus(ref, pointer, value);
	} else {
		USBExpertStatusLevel(level, ref, pointer, value);
	}
}

/************************************************************************************/
//
//	immediateError
//
//	Determines whether it's an error or just pending.
//
/************************************************************************************/

static Boolean immediateError(OSStatus err)
{
	return ((err != kUSBPending) && (err != noErr));
}

/************************************************************************************/
//
//	ConfigurationHandler
//
//	Configures the USB Serial Device (Modem).
//
/************************************************************************************/

static void ConfigurationHandler(USBPB *pb)
{
serPB 		*sp = &gSerialGlobals;

//	TraceMessage(0, kCRMName"- Entering ConfigurationHandler");
	StatusMessage(sp->deviceRef, kCRMName"- Entering ConfigurationHandler", pb->usbRefcon);
	
	if(pb->usbStatus != noErr)
	{
		if((sp->onError == kReset) && (sp->retries > 0))
		{
			/* no idea what to do now?? */
			USBExpertFatalError(sp->deviceRef, pb->usbStatus, sp->errorString, pb->usbRefcon);
			
			/* Mark port as errored */
			configured = true;
			configerr = pb->usbStatus;
		}
		else
		{
			StatusMessage(sp->deviceRef, sp->errorString, pb->usbStatus);
			pb->usbRefcon = sp->onError;

			USBClearPipeStallByReference(pb->usbReference);
			
			/* we'll delay coming back to here */
			sp->retries--;

			pb->usbReqCount = 0;
			USBDelay(pb);
		}		
		return;
	}

	sp->onError = kReset;
	
	do{switch(pb->usbRefcon++)
	{
		case kCommConfig:
		
			/* Need to find an interface with a communication class in it */
			pb->usbClassType = kUSBCommClass;
			pb->usbSubclass = 2;						/* Abstract Control Model */
			pb->usbProtocol = 1;						/* v.25ter (Hayes AT Command set) */
			pb->usb.cntl.WValue = 0;
			pb->usb.cntl.WIndex = 0;
			pb->usbReqCount = 0;
			pb->usbBuffer = nil;
			pb->usbFlags = 0;
			
			if (sp->retries > 0)
			{
				sp->onError = kCommConfig;
			} else {
				sp->onError = kReset;
			}			
			noteError(kCRMName"- Finding the Comm Class Interface");			
			if(immediateError(configerr = USBFindNextInterface(pb)))
			{
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;

		case kSetConfig:
			StatusMessage(sp->deviceRef, kCRMName"- Configuration Number - ", pb->usb.cntl.WValue);
			StatusMessage(sp->deviceRef, kCRMName"- Interface Number - ", pb->usb.cntl.WIndex);
			
			/* Remember the interface number */
			sp->interfacenum = pb->usb.cntl.WIndex;
			
			/* Open the device based on the config found */			
			if (sp->retries > 0)
			{
				sp->onError = kSetConfig;
			} else {
				sp->onError = kReset;
			}
			noteError(kCRMName"- Setting the configuration");
			if(immediateError(configerr = USBSetConfiguration(pb)))
			{
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
		
		break;
		
		case kGetCommInterface:
			StatusMessage(sp->deviceRef, kCRMName"- Number of interfaces in Configuration - ", pb->usbOther);
			
			/* Get the interface reference number */
			if (sp->retries > 0)
			{
				sp->onError = kGetCommInterface;
			} else {
				sp->onError = kReset;
			}
			pb->usbOther = 0;
			
			noteError(kCRMName"- Getting Comm Class interface");			
			if(immediateError(configerr = USBNewInterfaceRef(pb)))
			{
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
		
		break;
	
		case kSetCommInterface:
			StatusMessage(sp->deviceRef, kCRMName"- Comm. Interface Ref - ", pb->usbReference);
			
			/* Set the interface (Comm) */
			pb->usb.cntl.BMRequestType = USBMakeBMRequestType(kUSBNone, kUSBStandard, kUSBInterface);						
			pb->usb.cntl.BRequest = kUSBRqSetInterface;
			pb->usb.cntl.WValue = 0;
			pb->usb.cntl.WIndex = sp->interfacenum;
			pb->usbReqCount = 0;
			pb->usbBuffer = nil;
			if (sp->retries > 0)
			{
				sp->onError = kSetCommInterface;
			} else {
				sp->onError = kReset;
			}

			noteError(kCRMName"- Setting Comm Class interface");
			if(immediateError(configerr = USBDeviceRequest(pb)))
			{
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
			}
			break;

		case kConfigureCommInterface:
			
			/* configure the Comm Class interface */
			if (sp->retries > 0)
			{
				sp->onError = kConfigureCommInterface;
			} else {
				sp->onError = kReset;
			}
			
			noteError(kCRMName"- Configuring the Comm Class interface");
			if(immediateError(configerr = USBConfigureInterface(pb)))
			{	
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;
		
		case kGetInterruptEndpoint:
			StatusMessage(sp->deviceRef, kCRMName"- Number of pipes in Interface - ", pb->usbOther);
			
			/* Get the Comm Class interrupt endpoint */
			pb->usbFlags = kUSBIn;
			pb->usbClassType = kUSBInterrupt;
			if (sp->retries > 0)
			{
				sp->onError = kGetInterruptEndpoint;
			} else {
				sp->onError = kReset;
			}
			
			noteError(kCRMName"- Finding the interrupt endpoint");
			if(immediateError(configerr = USBFindNextPipe(pb)))
			{	
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;
		
		case kDataConfig:
			StatusMessage(sp->deviceRef, kCRMName"- Interrupt In Ref - ", pb->usbReference);
			
			/* Interrupt Endpoint is open - remember the ref */
			sp->interrupt = pb->usbReference;
			
			/* Need to find an interface with a data class in it */
			pb->usbClassType = kUSBDataClass;
			pb->usbSubclass = 0;
			pb->usbProtocol = 0;
			pb->usb.cntl.WValue = 0;
			pb->usb.cntl.WIndex = 0;
			pb->usbReqCount = 0;
			pb->usbBuffer = nil;
			pb->usbFlags = 0;
			pb->usbOther = 0;
			pb->usbReference = sp->deviceRef;		// Set it back to the device
			if (sp->retries > 0)
			{
				sp->onError = kDataConfig;
			} else {
				sp->onError = kReset;
			}

			noteError(kCRMName"- Finding the Data Interface");
			if(immediateError(configerr = USBFindNextInterface(pb)))
			{
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;
		
		case kGetDataInterface:
			StatusMessage(sp->deviceRef, kCRMName"- Configuration Number - ", pb->usb.cntl.WValue);
			StatusMessage(sp->deviceRef, kCRMName"- Interface Number - ", pb->usb.cntl.WIndex);
			
			/* Remember the interface number */
			sp->interfacenum = pb->usb.cntl.WIndex;
			
			/* Get the interface reference number */
			if (sp->retries > 0)
			{
				sp->onError = kGetDataInterface;
			} else {
				sp->onError = kReset;
			}			
			pb->usbOther = 0;
			
			noteError(kCRMName"- Getting Data Class interface");
			if(immediateError(configerr = USBNewInterfaceRef(pb)))
			{
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
		
		break;

		case kConfigureDataInterface:
			StatusMessage(sp->deviceRef, kCRMName"- Data Interface Ref - ", pb->usbReference);
			
			/* configure the Data Class interface */
			if (sp->retries > 0)
			{
				sp->onError = kConfigureDataInterface;
			} else {
				sp->onError = kReset;
			}
			
			noteError(kCRMName"- Configuring the Data Class interface");
			if(immediateError(configerr = USBConfigureInterface(pb)))
			{	
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;
			
		case kGetBulkOutEndpoint:
			StatusMessage(sp->deviceRef, kCRMName"- Number of pipes in Interface - ", pb->usbOther);
			
			/* Find the out endpoint */
			pb->usbFlags = kUSBOut;
			pb->usbClassType = kUSBBulk;
			pb->usbSubclass = 0;			/* Find the first one */
			if (sp->retries > 0)
			{
				sp->onError = kGetBulkOutEndpoint;
			} else {
				sp->onError = kReset;
			}
			
			noteError(kCRMName"- Finding the bulk out endpoint");
			if(immediateError(configerr = USBFindNextPipe(pb)))
			{	
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;
	
		case kGetBulkInEndpoint:
			StatusMessage(sp->deviceRef, kCRMName"- Bulk Out Endpoint - ", pb->usbReference);
			
			/* Out Endpoint is open - remember the ref */
			sp->bulkOut = pb->usbReference;
			
			/* Find the in endpoint */
			pb->usbFlags = kUSBIn;
			pb->usbClassType = kUSBBulk;
			pb->usbSubclass = 0;			/* Find the first one */
			if (sp->retries > 0)
			{
				sp->onError = kGetBulkInEndpoint;
			} else {
				sp->onError = kReset;
			}
			
			noteError(kCRMName"- Finding the bulk in endpoint");
			if(immediateError(configerr = USBFindNextPipe(pb)))
			{	
				USBExpertFatalError(sp->deviceRef, configerr, sp->errorString, 0);
				configured = true;
			}
			
		break;
	
		case kConfigDone:
			StatusMessage(sp->deviceRef, kCRMName"- Bulk In Endpoint - ", pb->usbReference);
			
			/* In Endpoint is open - remember the ref */
			sp->bulkIn = pb->usbReference;
			
			StartStatusMonitor(sp->interrupt);
			
			// device is fully configured, ready for somebody to set the baud rate, etc.
			// and start reading/writing to the bulk endpoints
			syncPB.usbStatus = kAvailableStatus;
			StatusMessage(sp->deviceRef, kCRMName"- Configuration complete.", 0);
			configured = true;
		break;
	
		default:
			noteError(kCRMName"- Internal Error unused case in Configuration handler");			
			StatusMessage(sp->deviceRef, sp->errorString, (pb->usbRefcon-1));
			configured = true;
			configerr = -1;
		break;
	}
	break;	/* only execute once, unless continue used */
	}while(1);	/* so case can be reentered with a continue */
}

UInt16	statusData[8];

/************************************************************************************/
//
//	ResetInterruptPB
//
//	Re-initialize the interrupt PB.
//
/************************************************************************************/

static void ResetInterruptPB(USBPB *pb)
{
	pb->usbReqCount = 16;
	pb->usbBuffer = &statusData;
	pb->usbStatus = noErr;
}

UInt8 	Intrretries = 0;

/************************************************************************************/
//
//	interruptCompletion
//
//	Interrupt completion handler.
//
/************************************************************************************/

static void interruptCompletion(USBPB *pb)
{
	serPB 		*sp = &gSerialGlobals;
	OSStatus	err;
	unsigned char		newErrs = 0;
	ShimSerialGlobals	*globals = gGlobals;
	
	TraceMessage(0, kCRMName"- Entering interruptCompletion");
	
	if (pb->usbStatus != kUSBAbortedError)	// are we being asked to quit?
	{
		if (pb->usbStatus != noErr)
		{
			StatusMessage(sp->deviceRef, kCRMName"- ModemDriver: interruptCompletion error ", pb->usbStatus);
			if (pb->usbStatus == kUSBEndpointStallErr)
			{
				ClearDevice();
			}
			USBClearPipeStallByReference(gSerialGlobals.interrupt);
			
			Intrretries++;
			if (Intrretries > 10)
				return;					// we'll just give up for now
				
			DoDelay();

		} else {
			Intrretries = 0;
			TraceMessage(0, kCRMName"- Interrupt received");
			if ((pb->usbActCount > 2) && ((statusData[0] & 0x00FF) == kSerialState))
			{
				sp->modemUSBStatus = HostToUSBWord(statusData[4]);
				
				if (sp->modemUSBStatus & kUSBParityErr) 
					newErrs |= parityErr;
				if (sp->modemUSBStatus & kUSBFramingErr) 
					newErrs |= framingErr;
				if (sp->modemUSBStatus & kUSBHwOverRunErr) 
					newErrs |= hwOverrunErr;
					
				globals->serStat.cumErrs |= newErrs;
				StatusMessage(sp->deviceRef, kCRMName"- Modem Serial state - ", sp->modemUSBStatus);
			}
		}
		
		ResetInterruptPB(pb);

		if(immediateError(err = USBIntRead(pb)))
		{
			StatusMessage(sp->deviceRef, kCRMName"- Couldn't queue interrupt read!", err);
		}
	}
}

/************************************************************************************/
//
//	StartStatusMonitor
//
//	Kick off the interrupt mechanism.
//
/************************************************************************************/

static void StartStatusMonitor(USBPipeRef interruptPipe)
{
	USBPB		*pb = &interruptPB;
	serPB	 	*sp = &gSerialGlobals;
	OSStatus	err;
	
	TraceMessage(0, kCRMName"- Entering StartStatusMonitor");
	
	InitializePB(pb, interruptPipe, interruptCompletion);

	ResetInterruptPB(pb);

	if(immediateError(err = USBIntRead(pb)))
	{
		StatusMessage(sp->deviceRef, kCRMName"- Couldn't start interrupt read!", err);
	}		

}

/************************************************************************************/
//
//	syncCompletion
//
//	Completion handler for all sync (setup) requests.
//
/************************************************************************************/

static void syncCompletion(USBPB *pb)
{
	serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering syncCompletion");

	if (pb->usbStatus != noErr)
	{
		USBExpertFatalError(sp->deviceRef, pb->usbStatus, kCRMName"- ModemDriver: syncCompletion err for bRequest", pb->usb.cntl.BRequest);
		if (pb->usbStatus == kUSBEndpointStallErr)
		{
//			ClearDevice();
		}
		USBClearPipeStallByReference(pb->usbReference);
	}
		
	pb->usbStatus = kAvailableStatus;
}

/************************************************************************************/
//
//	TimeoutPrevRequest
//
//	Timeout for all sync. requests.
//
/************************************************************************************/

Boolean TimeoutPrevRequest(void)
{
	USBPB		*pb = &syncPB;
	serPB	 	*sp = &gSerialGlobals;
	AbsoluteTime	startTime;
	Duration		elapsedTime;

	if (pb->usbStatus == kAvailableStatus)
		return false;
	
	startTime = UpTime();

	while (pb->usbStatus != kAvailableStatus){
		elapsedTime = AbsoluteDeltaToDuration(UpTime(), startTime);
		if (elapsedTime < 0) elapsedTime = elapsedTime/(-1000);  	// make sure it's in milliseconds
		if (elapsedTime > 100*durationMillisecond){
			StatusMessage(sp->deviceRef, kCRMName"- Modem driver, device request timeout - aborting", 0);
			USBAbortPipeByReference(pb->usbReference);
			return true;
		}
	}
	return false;
}

/************************************************************************************/
//
//	TimeoutStallRequest
//
//	Timeout for all stall clear requests.
//
/************************************************************************************/

Boolean TimeoutStallRequest(void)
{
	USBPB			*pb = &stallPB;
	serPB	 		*sp = &gSerialGlobals;
	AbsoluteTime	startTime;
	Duration		elapsedTime;

	if (pb->usbStatus == kAvailableStatus)
		return false;
	
	startTime = UpTime();

	while (pb->usbStatus != kAvailableStatus){
		elapsedTime = AbsoluteDeltaToDuration(UpTime(), startTime);
		if (elapsedTime < 0) elapsedTime = elapsedTime/(-1000);  	// make sure it's in milliseconds
		if (elapsedTime > 100*durationMillisecond){
			StatusMessage(sp->deviceRef, kCRMName"- Modem driver, Stall timeout - aborting", 0);
			USBAbortPipeByReference(pb->usbReference);
			return true;
		}
	}
	return false;
}

/************************************************************************************/
//
//	TimeoutDelayRequest
//
//	Timeout for delay requests.
//
/************************************************************************************/

Boolean TimeoutDelayRequest(void)
{
	USBPB			*pb = &delayPB;
	serPB	 		*sp = &gSerialGlobals;
	AbsoluteTime	startTime;
	Duration		elapsedTime;

	if (pb->usbStatus == kAvailableStatus)
		return false;
	
	startTime = UpTime();

	while (pb->usbStatus != kAvailableStatus){
		elapsedTime = AbsoluteDeltaToDuration(UpTime(), startTime);
		if (elapsedTime < 0) elapsedTime = elapsedTime/(-1000);  	// make sure it's in milliseconds
		if (elapsedTime > 100*durationMillisecond){
			return true;
		}
	}
	return false;
}

/************************************************************************************/
//
//	USBSetBaudRate
//
//	Set up and send SetLineCoding request for baud rate only.
//
/************************************************************************************/

void USBSetBaudRate(UInt32 baudRate)
{
	USBPB		*pb = &syncPB;
	serPB	 	*sp = &gSerialGlobals;
	OSStatus 	err;
	UInt16		temp = 0;
	UInt16		rateh;
	Boolean		setdte = false;
	
	TraceMessage(0, kCRMName"- Entering USBSetBaudRate");
	
	if (TimeoutPrevRequest())
		return;
	
	if (baudRate == kMaxBaudRate)		// We're going to lock to max for the moment
	{
	
		// convert baudrate			I hate Intel...
		temp = baudRate % 0x10000;
		rateh = USBToHostWord(temp);
		if (rateh != sp->Line_Settings.DTERate1)
		{
			sp->Line_Settings.DTERate1 = rateh;
			setdte = true;
		}
		temp = baudRate / 0x10000;
		rateh = USBToHostWord(temp);
		if (rateh != sp->Line_Settings.DTERate2)
		{
			sp->Line_Settings.DTERate2 = rateh;
			setdte = true;
		}
	
		// Only do it if it's changing
		if (setdte)
		{		
			pb->usbStatus = noErr;
			pb->pbVersion = kUSBCurrentPBVersion;
			pb->usb.cntl.BMRequestType = USBMakeBMRequestType(kUSBOut, kUSBClass, kUSBInterface);	
			pb->usb.cntl.BRequest = kSetLineCoding;
			pb->usb.cntl.WValue = 0;
			pb->usb.cntl.WIndex = 1; 
			pb->usbReqCount = sizeof(sp->Line_Settings)-1;
			pb->usbBuffer = &sp->Line_Settings;

			if(immediateError(err = USBDeviceRequest(pb)))
			{
				StatusMessage(sp->deviceRef, kCRMName"- Modem driver Setting baud rate", err);
				pb->usbStatus = kAvailableStatus;
			}
		}
	}
}

/************************************************************************************/
//
//	USBSetLineCoding
//
//	Set up and send SetLineCoding Request for all settings.
//
/************************************************************************************/

void USBSetLineCoding(LineParms Line_Coding)
{
	USBPB		*pb = &syncPB;
	serPB	 	*sp = &gSerialGlobals;
	OSStatus 	err;
	Boolean		setcoding = false;
	
	TraceMessage(0, kCRMName"- Entering USBSetLineCoding");
	
	if (TimeoutPrevRequest())
		return;
		
	// save the settings only if they've changed
	if (Line_Coding.CharFormat != sp->Line_Settings.CharFormat)
	{
		sp->Line_Settings.CharFormat = Line_Coding.CharFormat;
		setcoding = true;
	}
	if (Line_Coding.ParityType != sp->Line_Settings.ParityType)
	{
		sp->Line_Settings.ParityType = Line_Coding.ParityType;
		setcoding = true;
	}
	if (Line_Coding.DataBits != sp->Line_Settings.DataBits)
	{
		sp->Line_Settings.DataBits = Line_Coding.DataBits;
		setcoding = true;
	}
	
	// Don't do this if they haven't changed
	if (setcoding)
	{
		pb->usbStatus = noErr;
		pb->pbVersion = kUSBCurrentPBVersion;
		pb->usb.cntl.BMRequestType = USBMakeBMRequestType(kUSBOut, kUSBClass, kUSBInterface);				
		pb->usb.cntl.BRequest = kSetLineCoding;
		pb->usb.cntl.WValue = 0;
		pb->usb.cntl.WIndex = 1; 
		pb->usbReqCount = sizeof(sp->Line_Settings)-1;
		pb->usbBuffer = &sp->Line_Settings;

		if(immediateError(err = USBDeviceRequest(pb)))
		{
			StatusMessage(sp->deviceRef, kCRMName"- Modem driver Setting Line Coding", err);
			pb->usbStatus = kAvailableStatus;
		}
	}
}

/************************************************************************************/
//
//	USBSetControlLineState
//
//	Set up and send SetControlLineState (DTR and RTS).
//
/************************************************************************************/

void USBSetControlLineState(void)
{
	USBPB		*pb = &syncPB;
	serPB	 	*sp = &gSerialGlobals;
	OSStatus 	err;
	
	TraceMessage(0, kCRMName"- Entering USBSetControlLineState");
	
	if (TimeoutPrevRequest())
		return;

	pb->usbStatus = noErr;
	pb->pbVersion = kUSBCurrentPBVersion;
	pb->usb.cntl.BMRequestType = USBMakeBMRequestType(kUSBOut, kUSBClass, kUSBInterface);
	pb->usb.cntl.BRequest = kSetControlLineState;
	pb->usb.cntl.WValue = sp->lineState;
	pb->usb.cntl.WIndex = 1; 
	pb->usbReqCount = 0;
	pb->usbBuffer = 0;

	if(immediateError(err = USBDeviceRequest(pb)))
	{
		StatusMessage(sp->deviceRef, kCRMName"- Modem driver Setting Control Line State", err);
		pb->usbStatus = kAvailableStatus;
	}
}

/************************************************************************************/
//
//	USBSetDTRState
//
//	Set the DTR state.
//
/************************************************************************************/

void USBSetDTRState(Boolean state)
{
	serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering USBSetDTRState");

	if (state)
	{
		sp->lineState |= kDTROn;
	} else {
		if (!sp->DTRClose)
			sp->lineState &= (kDTROff + kRTSOn);
	}
}

/************************************************************************************/
//
//	USBSetRTSState
//
//	Set the RTS state.
//
/************************************************************************************/

void USBSetRTSState(Boolean state)
{
	serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering USBSetRTSState");

	if (state)
	{
		sp->lineState |= kRTSOn;
	} else {
		sp->lineState &= (kRTSOff + kDTROn);
	}
}

/************************************************************************************/
//
//	USBSendBreak
//
//	Set up and send break request.
//
/************************************************************************************/

void USBSendBreak(Boolean state)
{
	USBPB		*pb = &syncPB;
	serPB	 	*sp = &gSerialGlobals;
	OSStatus 	err;
	
	TraceMessage(0, kCRMName"- Entering USBSendBreak");
	
	if (TimeoutPrevRequest())
		return;

	pb->usbStatus = noErr;
	pb->pbVersion = kUSBCurrentPBVersion;
	pb->usb.cntl.BMRequestType = USBMakeBMRequestType(kUSBOut, kUSBClass, kUSBInterface);
	pb->usb.cntl.BRequest = kSendBreak;
	if (state)
	{
		pb->usb.cntl.WValue = kBreakOn;
	} 
	else 
	{
		pb->usb.cntl.WValue = kBreakOff;
	}
	pb->usb.cntl.WIndex = 1; 
	pb->usbReqCount = 0;
	pb->usbBuffer = 0;

	if(immediateError(err = USBDeviceRequest(pb)))
	{
		StatusMessage(sp->deviceRef, kCRMName"- Modem driver Setting Break State", err);
		pb->usbStatus = kAvailableStatus;
	}
}

/************************************************************************************/
//
//	USBSetCloseDTR
//
//	Set the state of DTR on close.
//
/************************************************************************************/

void USBSetCloseDTR(void)
{
	serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering USBSetCloseDTR");

	sp->DTRClose = true;
}

/************************************************************************************/
//
//	USBGetDCDValue
//
//	Gets the current state of CD.
//
/************************************************************************************/

UInt8 USBGetDCDValue(void)
{
	serPB 	*sp = &gSerialGlobals;
	UInt8	dcd = 0;

	TraceMessage(0, kCRMName"- Entering USBGetDCDValue");

	if (sp->modemUSBStatus & kUSBDCD) 
		dcd = 1;
		
	return dcd;
}

/************************************************************************************/
//
//	ModemDriverEntry
//
//	Initializes and starts the whole show.
//
/************************************************************************************/

OSStatus modemDriverEntry(USBDeviceRef device, USBDeviceDescriptor *desc)
{
static Boolean 				beenThereDoneThat = false;
static USBDeviceDescriptor 	ourDeviceDescriptor;

//	DebugMessage(kCRMName"- Entering modemDriverEntry - Sync. point");
	TraceMessage(0, kCRMName"- Entering modemDriverEntry");
		
	if(beenThereDoneThat)
	{
		StatusMessage(device, kCRMName"- Modem driver called second time", 0);
		return -1;
	}
	beenThereDoneThat = true;
	
	ourDeviceDescriptor = *desc;
	gSerialGlobals.deviceDescriptor = &ourDeviceDescriptor;
	gSerialGlobals.deviceRef = device;
	
	InitializePB(&syncPB, device, syncCompletion);
	InitializePB(&stallPB, device, stallHandler);
	stallPB.usbStatus = kAvailableStatus;
	InitializePB(&delayPB, device, delayHandler);
	delayPB.usbStatus = kAvailableStatus;

	InitializePB(&gSerialGlobals.pb, device, ConfigurationHandler);
	gSerialGlobals.pb.usbRefcon = kCommConfig;	// Where we start from (kReset used for errors)
	gSerialGlobals.pb.usbBuffer = nil;

	gSerialGlobals.retries = 3;				// Let's keep it reasonable for now
	
	CheckUSBVersion();						// Establish version and bulk fix flag before we start
	
	ConfigurationHandler(&gSerialGlobals.pb);
	
	if (configerr == 1)						// pending's ok
	{
		return noErr;
	} else {
		return configerr;
	}
}

/************************************************************************************/
//
//	InitializePB
//
//	Initial a parameter block.
//
/************************************************************************************/

static void InitializePB(USBPB *pb, USBDeviceRef ref, USBCompletion handler)
{
	pb->pbVersion = kUSBCurrentPBVersion;
	pb->pbLength = sizeof(*pb);
	pb->usbReference = ref;
	pb->usbCompletion = handler;
	pb->usbStatus = noErr;
	
}

static errCount, readActive = false;

/************************************************************************************/
//
//	readCompletion
//
//	Completion handler for USB reads.
//
/************************************************************************************/

void readCompletion(USBPB *pb)
{

	TraceMessage(0,kCRMName"- Entering readCompletion");
	
	readActive = false;
	
	if (pb->usbStatus != kUSBAbortedError)	// are we being asked to quit?
	{
		if (pb->usbStatus == noErr)
		{
			LogData(kUSBIn, pb->usbActCount, pb->usbBuffer);
			HAL_ShimInput(pb->usbBuffer, pb->usbActCount);
		} else {
			StatusMessage(gSerialGlobals.deviceRef, kCRMName"- Read completion error ", pb->usbStatus);
				
			if ((pb->usbStatus != kUSBUnderRunErr) && errCount++ < 10)
				StatusMessage(gSerialGlobals.deviceRef, kCRMName"- Read completion error ", pb->usbStatus);
				
			if (pb->usbStatus == kUSBEndpointStallErr)
			{
//				ClearDevice();
			}
			USBClearPipeStallByReference(gSerialGlobals.bulkIn);
			if (pb->usbActCount > 0)
			{
				LogData(kUSBIn, pb->usbActCount, pb->usbBuffer);
				HAL_ShimInput(pb->usbBuffer, pb->usbActCount);
			}
		}
	
		USBStartReadPolling();
	}
}

static UInt8	ioBuffer[64];

/************************************************************************************/
//
//	USBStartReadPolling
//
//	Start the USB read mechanism.
//
/************************************************************************************/

void USBStartReadPolling(void)
{
	OSStatus status;
	
	TraceMessage(0,kCRMName"- Entering USBStartReadPolling");
	
	if (gSerialGlobals.bulkIn && !readActive){
		InitializePB(&readPB, gSerialGlobals.bulkIn, readCompletion);
		readPB.usbBuffer = ioBuffer;
		readPB.usbReqCount = sizeof(ioBuffer);
		if(immediateError(status = SafeUSBBulkRead(&readPB)))
//		if(immediateError(status = USBBulkRead(&readPB)))
		{
			USBExpertFatalError(gSerialGlobals.deviceRef, status, kCRMName"- ModemDriver: Couldn't start read polling", 0);
			return;
		}
		readActive = true;
	}
}

/************************************************************************************/
//
//	USBStopReadPolling
//
//	You guessed it stop the USB read mechanism.
//
/************************************************************************************/

void USBStopReadPolling()
{
	StatusMessage(gSerialGlobals.deviceRef, kCRMName"- Aborting Bulk-in pipe", 0);

	if (gSerialGlobals.bulkIn){
		USBAbortPipeByReference(gSerialGlobals.bulkIn);
	}
}

static	UInt32	writeActive;

/************************************************************************************/
//
//	writeCompletion
//
//	USB write completion handler.
//
/************************************************************************************/

void writeCompletion(USBPB *pb)
{
	IOParam	*iopb;
	
	TraceMessage(0, kCRMName"- Entering writeCompletion");
	
	if (pb->usbStatus != kUSBAbortedError)	// are we being asked to quit?
	{
		if (pb->usbStatus != noErr) 
		{
			StatusMessage(gSerialGlobals.deviceRef, kCRMName"- Write Failed", pb->usbStatus);
			
			if (pb->usbStatus == kUSBEndpointStallErr)
			{
//				ClearDevice();
			}
			USBClearPipeStallByReference(pb->usbReference);
		}
	}
	writeActive--;
	gGlobals->pbOut = nil;
	iopb = (IOParam *)pb->usbRefcon;
	iopb->ioActCount = pb->usbActCount;
	
	ShimIOComplete((union ParamBlockRec *)iopb, pb->usbStatus ? ioErr : 0);

}

/************************************************************************************/
//
//	USBSerialWrite
//
//	USB serial write routine.
//
/************************************************************************************/

OSStatus USBSerialWrite(IOParam *pb)
{
	OSStatus 	status = noErr;
	
	TraceMessage(0, kCRMName"- Entering USBSerialWrite");
	
	if (writeActive){
		StatusMessage(gSerialGlobals.deviceRef, kCRMName"- Overlapping writes!", writePB.usbStatus);
		status = ioErr;
	} else if (gSerialGlobals.bulkOut){
		
		InitializePB(&writePB, gSerialGlobals.bulkOut, writeCompletion);
		writePB.usbRefcon = (UInt32)pb;
		writePB.usbBuffer = pb->ioBuffer;
		writePB.usbReqCount = pb->ioReqCount;
		
		LogData(kUSBOut, writePB.usbReqCount, writePB.usbBuffer);
		
		if(immediateError(status = SafeUSBBulkWrite(&writePB)))
//		if(immediateError(status = USBBulkWrite(&writePB)))
		{
			StatusMessage(gSerialGlobals.deviceRef, kCRMName"- ModemDriver: Couldn't start write", status);
		} else {
			writeActive++;
			status = 1;						// pending
		}
	} else {
		status = ioErr;
	}
	return status;
}

/************************************************************************************/
//
//	KillUSBIO
//
//	Kill all USB io operations in progress.
//
/************************************************************************************/

void KillUSBIO(void)
{
	IOParam 			*pb;
	ShimSerialGlobals 	*globals = gGlobals;
	
	StatusMessage(gSerialGlobals.deviceRef, kCRMName"- Modem: Killing all USB IO", 0);

	if (gSerialGlobals.bulkIn){
		USBAbortPipeByReference(gSerialGlobals.bulkIn);
		gSerialGlobals.bulkIn = nil;
	}
	if (gSerialGlobals.bulkOut){
		USBAbortPipeByReference(gSerialGlobals.bulkOut);
		gSerialGlobals.bulkOut = nil;
	}
	if (gSerialGlobals.interrupt){
		USBAbortPipeByReference(gSerialGlobals.interrupt);
		gSerialGlobals.interrupt = nil;
	}
	
	pb = (IOParam *)globals->pbIn;
	if (pb) {
		ShimIOComplete((union ParamBlockRec *)pb, ioErr);
		globals->pbIn = nil;
	}
	pb = (IOParam *)globals->pbOut;
	if (pb) {
		ShimIOComplete((union ParamBlockRec *)pb, ioErr);
		globals->pbOut = nil;
	}
}

/************************************************************************************/
//
//	InitLineCoding
//
//	Store the initial line settings.
//
/************************************************************************************/

void InitLineCoding(LineParms Line_Coding)
{
	serPB 	*sp = &gSerialGlobals;
	
	TraceMessage(0, kCRMName"- Entering InitLineCoding");
		
	// save the settings
	sp->Line_Settings.DTERate1 = Line_Coding.DTERate1;
	sp->Line_Settings.DTERate2 = Line_Coding.DTERate2;
	sp->Line_Settings.CharFormat = Line_Coding.CharFormat;
	sp->Line_Settings.ParityType = Line_Coding.ParityType;
	sp->Line_Settings.DataBits = Line_Coding.DataBits;
}

/************************************************************************************/
//
//	DoDelay
//
//	Set up to do a USB delay.
//
/************************************************************************************/

void DoDelay(void)
{
serPB 	*sp = &gSerialGlobals;
	
	TraceMessage(0, kCRMName"- Entering DoDelay");
	
	if (TimeoutDelayRequest())
		return;
			
	delayPB.usbStatus = noErr;
	delayPB.usbReference = sp->deviceRef;
	delayPB.usbRefcon = 1;
	delayPB.usbReqCount = 5;			// ~5ms (actually 5 frames at 1ms each)
	USBDelay(&delayPB);	
}

/************************************************************************************/
//
//	delayHandler
//
//	Completion handler for USB delay.
//
/************************************************************************************/

static void delayHandler(USBPB *pb)
{
serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering delayHandler");

	if (pb->usbStatus != noErr)
		StatusMessage(sp->deviceRef, kCRMName"- ModemDriver: Delay fail?", pb->usbStatus);

	pb->usbStatus = kAvailableStatus;
}

/************************************************************************************/
//
//	ClearDevice
//
//	Set up and send ClearFeature request (for stall).
//
/************************************************************************************/

void ClearDevice(void)
{
serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering ClearDevice");
	
	if (TimeoutStallRequest())
		return;
		
	stallPB.usbStatus = noErr;
	stallPB.usbReference = sp->deviceRef;
	stallPB.usbRefcon = kEndpointStall;
	stallHandler(&stallPB);
}

/************************************************************************************/
//
//	stallHandler
//
//	Completion handler for clear device/stall.
//
/************************************************************************************/

static void stallHandler(USBPB *pb)
{
serPB	 	*sp = &gSerialGlobals;
OSStatus	err = 0;

	TraceMessage(0, kCRMName"- Entering StallHandler");
	
	if (pb->usbStatus != noErr)
	{
		USBExpertFatalError(sp->deviceRef, pb->usbStatus, kCRMName"- Driver: Stall fail?", pb->usbRefcon);
		return;
	}
	
	// May have to do more here (like find out which endpoint is stalled) which is why an FSM is used
	// For the moment it's the Comm Class endpoint 0 which we'll try to clear (interupptComplete only for now)
	do{switch(pb->usbRefcon++)
	{
		case kEndpointStall:
			pb->usbStatus = noErr;
			pb->usbReference = sp->deviceRef;
			pb->pbVersion = kUSBCurrentPBVersion;	
			pb->usb.cntl.BMRequestType = USBMakeBMRequestType(kUSBOut, kUSBStandard, kUSBEndpoint);
	
			pb->usb.cntl.BRequest = kUSBRqClearFeature;
			pb->usb.cntl.WValue = 0;						// Endpoint stall
			pb->usb.cntl.WIndex = 0;
			pb->usbReqCount = 0;
			pb->usbBuffer = nil;

			if(immediateError(err = USBDeviceRequest(pb)))
			{
				StatusMessage(sp->deviceRef, kCRMName"- Driver clearing device stall error", err);
			}
		break;
		
		case kStallDone:
		// stall should now be cleared
			if (pb->usbStatus != noErr)
				USBExpertFatalError(sp->deviceRef, pb->usbStatus, kCRMName"- Driver: stall handler err for bRequest", pb->usb.cntl.BRequest);
	
			pb->usbStatus = kAvailableStatus;
		break;
		
		default:
			StatusMessage(sp->deviceRef, kCRMName"- Internal Error unused case in stall handler", (pb->usbRefcon-1));
		break;
	}
	break;	/* only execute once, unless continue used */
	}while(1);	/* so case can be reentered with a continue */
}

/************************************************************************************/
//
//	ResetDevice
//
//	Resets the device (uses same PB as ClearStall).
//
/************************************************************************************/

void ResetDevice(void)
{
serPB 	*sp = &gSerialGlobals;

	TraceMessage(0, kCRMName"- Entering ResetDevice");
	
	if (TimeoutStallRequest())
		return;
		
	stallPB.usbStatus = noErr;
	stallPB.usbReference = sp->deviceRef;
	stallPB.usbRefcon = kResetDevice;
	ResetHandler(&stallPB);
}

/************************************************************************************/
//
//	ResetHandler
//
//	Completion handler for Reset device.
//
/************************************************************************************/

static void ResetHandler(USBPB *pb)
{
serPB	 	*sp = &gSerialGlobals;
OSStatus	err = 0;

	TraceMessage(0, kCRMName"- Entering ResetHandler");
	
	if (pb->usbStatus != noErr)
	{
		USBExpertFatalError(sp->deviceRef, pb->usbStatus, kCRMName"- Driver: Reset fail?", pb->usbRefcon);
		return;
	}
	
	// May have to do a couple of things here 
	do{switch(pb->usbRefcon++)
	{
		case kResetDevice:
			pb->usbStatus = noErr;
			pb->usbReference = sp->deviceRef;
			pb->pbVersion = kUSBCurrentPBVersion;	
			pb->usb.cntl.WValue = 0;
			pb->usb.cntl.WIndex = 0;
			pb->usbReqCount = 0;
			pb->usbBuffer = nil;
			pb->usbFlags = 0;

			if(immediateError(err = USBResetDevice(pb)))
			{
				StatusMessage(sp->deviceRef, kCRMName"- Driver reset device error", err);
			}
		break;
		
		case kResetDone:
			if (pb->usbStatus != noErr)
				USBExpertFatalError(sp->deviceRef, pb->usbStatus, kCRMName"- Driver: reset handler err ", pb->usbRefcon);
	
			pb->usbStatus = kAvailableStatus;
		break;
		
		default:
			StatusMessage(sp->deviceRef, kCRMName"- Internal Error unused case in reset handler", (pb->usbRefcon-1));
		break;
	}
	break;	/* only execute once, unless continue used */
	}while(1);	/* so case can be reentered with a continue */
}

/************************************************************************************/
//
//	Asciify
//
//	Convert to Ascii character
//
/************************************************************************************/
 
UInt8 Asciify(UInt8 i)
{

	i &= 0xF;
	if ( i < 10 )
		 return( '0' + i );
	else return( 55  + i );
	
}

#if ((DebugOn > 0) && (LogOn > 0))

#define dumplen		16		// Set this to the number of bytes to dump and the rest should work out correct

#define buflen		((dumplen*2)+dumplen)+3
#define Asciistart	(dumplen*2)+3

/************************************************************************************/
//
//	USBLogData
//
//	Dumps the requested amount of data to the USB Expert log.
//
/************************************************************************************/

void USBLogData(UInt8 Dir, UInt32 Count, UInt8 *buf)
{
	UInt8		wlen, i, Aspnt, Hxpnt;
	UInt8		wchr;
	UInt8		LocBuf[buflen];

	for ( i=1; i<=buflen; i++)
	{
		LocBuf[i] = 0x20;
	}
	
	if (Dir == kUSBIn)
	{
		TraceMessage(1, kCRMName"- Read Complete");
	} else {
		if (Dir == kUSBOut)
		{
			TraceMessage(1, kCRMName"- Write");
		}
	}

	if (Count > dumplen)
	{
		wlen = dumplen;
	} else {
		wlen = Count;
	}
	
	if (wlen > 0)
	{
		Aspnt = Asciistart;
		Hxpnt = 1;
		for (i=1; i<=wlen; i++)
		{
			wchr = buf[i-1];
			LocBuf[Hxpnt++] = Asciify(wchr >> 4);
			LocBuf[Hxpnt++] = Asciify(wchr);
			if ((wchr < 0x20) || (wchr == 0x7F)) 		// Non printable characters
			{
				LocBuf[Aspnt++] = 0x2E;					// Replace with a period
			} else {
				LocBuf[Aspnt++] = wchr;
			}
		}
		LocBuf[0] = (wlen + Asciistart) + 1;
		TraceMessage(1, LocBuf);
	} else {
		TraceMessage(1, kCRMName"- No data - Actual count=0");
	}
}
#endif